//
//  GameView.swift
//  ArkitDoushiQi
//
//  Created by Enzo JOLYS on 27/05/2024.
//

import SwiftUI
import SpriteKit
import DouShouQiModel

struct GameView: View {
    
    var gameScene:GameScene = GameScene(size: CGSize(width: 940, height: 740))

    @State var msg:String = "Msg"
    
    var body: some View {

        VStack(){
            Text(msg)
            SpriteView(scene: gameScene).task {
 
            gameScene.game.addGameStartedListener { board in startGame()}
            gameScene.game.addGameOverListener { board, result, player in gameOver(result: result) }
            gameScene.game.addGameChangedListener { game in gameChange() }
            gameScene.game.addBoardChangedListener { board in boardChange() }

            gameScene.game.addMoveChosenCallbacksListener { board, move, player in moveChose(board: board, move: move, player: player) }
            gameScene.game.addInvalidMoveCallbacksListener { board,move,player,bool in invalidMove(board: board, move: move, player:player, bool:bool)}
                
            gameScene.game.addPieceRemovedListener { _,_,piece in removePiece(piece: piece) }
            
            gameScene.game.addPlayerNotifiedListener { board, player in
                print("Player notif : \(player.id) à toi de jouer ")
                msg = "Player notif : \(player.id) à toi de jouer !"
                
                if (player is IAPlayer ){
                    try! await player.chooseMove(in: board, with: gameScene.game.rules)
                }
            }
            
            try! await gameScene.game.start() 
            }
        }
    }
    
    // ------ Listener -------- //
    
    func startGame() { print("Start game !")
        msg = "Start !!"
    }
    
    func gameOver(result : Result) { print("Game over !")
        msg = "Game over ! "
        switch result
        {
        case .winner(let winner, _):
            switch winner {
            case .player1:
                msg = msg + "player 1 wins !"
            case .player2:
                msg = msg + "player 2 wins !"
            default:
                msg = msg + "equality !"
            }
        default:
            msg="Game over !"
        }
        
        gameScene.isGameOver = true
        actionEndGame(scene: gameScene,message: msg)
    }
    
    func gameChange() { print("Game change !") }
    func boardChange() {
        print("Board change !")
        gameScene.displayBoard(board: gameScene.game.board)
    }
    
    func moveChose(board:Board,move:Move,player:Player) {
    }
    
    func invalidMove(board:Board,move:Move,player:Player,bool:Bool) {
        
        if (bool){ // Valid
            print("Valid move of \(player.id)")
            print("Move : \(move.description)")
        }
        else { // Invalid
            print("Invalid move of \(player.id)")
            print("Move : \(move.description)")
            gameScene.displayBoard(board: gameScene.game.board)
        }
        print("------------")
    }
    
    func removePiece(piece:Piece){
        print("Remove piece")
        if let node = gameScene.pieces[piece.owner]![piece.animal] {
            actionRemove(size: gameScene.size, node: node, scene: gameScene)
            print("Remove piece from parent")
        }
    }
    
    
    // ------------------------- //
    
}

struct GameView_Previews: PreviewProvider {
    static var previews: some View {
        GameView()
    }
}
